@@ -117,7 +117,7 @@ static __always_inline void mwait_idle_with_hints(unsigned long eax, unsigned lo
 			mb();
 		}
 
-		if (irqs_disabled() &&
+		if (irqs_disabled() && (ecx & 1) &&
 		    cpu_feature_enabled(X86_FEATURE_KERNEL_IBRS)) {
 			/* NMI always enable IBRS on exception entry */
 			ibrs_disabled = true;
@@ -127,9 +127,15 @@ static __always_inline void mwait_idle_with_hints(unsigned long eax, unsigned lo
 		}
 
 		__monitor((void *)&current_thread_info()->flags, 0, 0);
-		if (!need_resched())
-			__mwait(eax, ecx);
 
+		if (!need_resched()) {
+			if (ecx & 1) {
+				__mwait(eax, ecx);
+			} else {
+				__sti_mwait(eax, ecx);
+				raw_local_irq_disable();
+			}
+		}
 		if (ibrs_disabled) {
 			native_wrmsrl(MSR_IA32_SPEC_CTRL, spec_ctrl);
 			__this_cpu_write(x86_spec_ctrl_current, spec_ctrl);
