@@ -7,11 +7,23 @@
  *
  */
 #include <linux/fb.h>
-#include <linux/pci.h>
 #include <linux/module.h>
+#include <linux/pci.h>
 #include <linux/vgaarb.h>
 #include <asm/fb.h>
 
+pgprot_t pgprot_framebuffer(pgprot_t prot,
+			    unsigned long vm_start, unsigned long vm_end,
+			    unsigned long offset)
+{
+	pgprot_val(prot) &= ~_PAGE_CACHE_MASK;
+	if (boot_cpu_data.x86 > 3)
+		pgprot_val(prot) |= cachemode2protval(_PAGE_CACHE_MODE_UC_MINUS);
+
+	return prot;
+}
+EXPORT_SYMBOL(pgprot_framebuffer);
+
 int fb_is_primary_device(struct fb_info *info)
 {
 	struct device *device = info->device;
@@ -27,4 +39,5 @@ int fb_is_primary_device(struct fb_info *info)
 	return 0;
 }
 EXPORT_SYMBOL(fb_is_primary_device);
+
 MODULE_LICENSE("GPL");
