@@ -383,7 +383,7 @@ static int scsi_complete_sghdr_rq(struct request *rq, struct sg_io_hdr *hdr,
 	 * fill in all the output members
 	 */
 	hdr->status = req->result & 0xff;
-	hdr->masked_status = status_byte(req->result);
+	hdr->masked_status = sg_status_byte(req->result);
 	hdr->msg_status = COMMAND_COMPLETE;
 	hdr->host_status = host_byte(req->result);
 	hdr->driver_status = 0;
